`timescale 1ns/1ps

module tb;

// GSR GSR(1'b1);

localparam      BIT_ONE             =   1'b1;
localparam      BIT_ZERO            =   1'b0;

//
reg             clk_in;
reg             rstn;
reg             pcs_tx_o_fabric_clk;
reg             pcs_rx_o_fabric_clk;

//IP configuration_vector
reg             basex_or_sgmii;
reg             sgmii_mode;
reg     [ 8:0]  basex_link_timer;
reg     [ 8:0]  sgmii_link_timer;

reg             main_reset;
reg             an_enable;
reg             an_restart;
reg     [ 2:0]  confi_vector;

reg             ip_link_complete;           // PHY LINK UP
reg             an_complete;
reg             an_complete_d1;
reg     [ 7:0]  ip_link_complete_cnt_tx;
reg             tx_mac_rst_n;
reg     [ 7:0]  ip_link_complete_cnt_rx;
reg             rx_mac_rst_n;
reg             auto_negotiation_complete;

// //IP TX PORT
wire            tx_mac_clk;
wire            tx_mac_valid;
wire    [15:0]  tx_mac_data;
wire            tx_mac_last;
wire            tx_mac_ub_invalid;
wire            tx_mac_error;
wire            tx_mac_ready;

wire            tx_pause_req;
wire    [15:0]  tx_pause_val;
wire    [47:0]  tx_pause_source_addr;

wire            tx_statistics_valid;

wire    [19:0]  serdes_txdata;
//IP RX PORT
wire            rx_mac_clk;
wire            rx_mac_valid;
wire    [15:0]  rx_mac_data;
wire            rx_mac_last;
wire            rx_mac_ub_invalid;
wire            rx_mac_error;

wire            rx_statistics_valid;
wire    [26:0]  rx_statistics_vector;
wire            rx_pause_req;
wire    [15:0]  rx_pause_val;

wire    [21:0]  serdes_rxdata;
//IP MDIO/MDC PORT
reg     [14:0]  configuration_vector;
wire    [10:0]  status_vector;
wire    [15:0]  link_partner_ability_base;

//CHECK RESULT
reg             cycle_one_time;
reg             cycle_all_time;
reg             length_hold;
reg             no_pause;
reg             check_clear;

wire            tx_running;
wire            tx_cnt_er;
wire    [31:0]  tx_cnt;

wire            error_all_data_check;
wire            error_all_pause_check;
wire            error_all_check;
wire            rx_cnt_er;
wire    [31:0]  rx_cnt;

reg     [15:0]  cycle_cnt;

Two_Giga_Serial_Ethernet_Top u_Two_Giga_Serial_Ethernet_Top(
        .serdes_fabric_ctrl_o           (                           ),
        .serdes_fabric_ctrl_h_o         (                           ),
        .serdes_fabric_rstn_o           (                           ),
        .serdes_fabric_tx_clk_o         (                           ),
        .serdes_fabric_rx_clk_o         (                           ),
        .serdes_pcs_tx_rst_o            (                           ),
        .serdes_pcs_rx_rst_o            (                           ),
        .serdes_fabric_tx_vld_o         (                           ),
        .serdes_rxfifo_rd_en_o          (                           ),
        .serdes_txdata_o                (serdes_txdata              ),
        .rx_mac_clk_o                   (rx_mac_clk                 ),
        .rx_mac_valid_o                 (rx_mac_valid               ),
        .rx_mac_data_o                  (rx_mac_data                ),
        .rx_mac_last_o                  (rx_mac_last                ),
        .rx_mac_ub_invalid_o            (rx_mac_ub_invalid          ),
        .rx_mac_error_o                 (rx_mac_error               ),
        .tx_mac_clk_o                   (tx_mac_clk                 ),
        .tx_mac_ready_o                 (tx_mac_ready               ),
        .tx_collision_o                 (                           ),
        .tx_retransmit_o                (                           ),
        .rx_statistics_valid_o          (rx_statistics_valid        ),
        .rx_statistics_vector_o         (rx_statistics_vector       ),
        .rx_pause_req_o                 (rx_pause_req               ),
        .rx_pause_val_o                 (rx_pause_val               ),
        .tx_statistics_valid_o          (tx_statistics_valid        ),
        .tx_statistics_vector_o         (                           ),
        .miim_rddata_o                  (                           ),
        .miim_rddata_valid_o            (                           ),
        .miim_busy_o                    (                           ),
        .mdc_o                          (                           ),
        .mdio_out_o                     (                           ),
        .mdio_oen_o                     (                           ),
        .status_vector_o                (status_vector              ),
        .link_partner_ability_base_o    (link_partner_ability_base  ),
        .clk_in_i                       (clk_in                     ),
        .rstn_i                         (rstn                       ),
        .serdes_pcs_tx_clk_i            (pcs_tx_o_fabric_clk        ),
        .serdes_pcs_rx_clk_i            (pcs_rx_o_fabric_clk        ),
        .serdes_qpll1_ok_i              (BIT_ONE                    ),
        .serdes_qpll0_ok_i              (BIT_ONE                    ),
        .serdes_cpll_ok_i               (BIT_ONE                    ),
        .serdes_pma_rx_lock_i           (BIT_ONE                    ),
        .serdes_tx_fifo_afull_i         (BIT_ZERO                   ),
        .serdes_rxfifo_aempty_i         (BIT_ZERO                   ),
        .serdes_rxdata_i                (serdes_rxdata              ),
        .signal_detect_i                (BIT_ONE                    ),
        .basex_link_timer_i             (basex_link_timer           ),
        .sgmii_link_timer_i             (sgmii_link_timer           ),
        .basex_or_sgmii_i               (basex_or_sgmii             ),
        .sgmii_mode_i                   (sgmii_mode                 ),
        .repeater_mode_i                (BIT_ZERO                   ),
        .tx_mac_valid_i                 (tx_mac_valid               ),
        .tx_mac_data_i                  (tx_mac_data                ),
        .tx_mac_last_i                  (tx_mac_last                ),
        .tx_mac_ub_invalid_i            (tx_mac_ub_invalid          ),
        .tx_mac_error_i                 (tx_mac_error               ),
        .rx_fcs_fwd_ena_i               (1'd0                       ),
        .rx_jumbo_ena_i                 (1'd0                       ),
        .tx_pause_req_i                 (tx_pause_req               ),
        .tx_pause_val_i                 (tx_pause_val               ),
        .tx_pause_source_addr_i         (tx_pause_source_addr       ),
        .tx_ifg_delay_ena_i             (1'd0                       ),
        .tx_ifg_delay_i                 (8'd0                       ),
        .tx_fcs_fwd_ena_i               (1'd0                       ),
        .miim_hs_clk_i                  (1'd0                       ),
        .miim_phyad_i                   (5'd0                       ),
        .miim_regad_i                   (5'd0                       ),
        .miim_wrdata_i                  (16'd0                      ),
        .miim_wren_i                    (1'd0                       ),
        .miim_rden_i                    (1'd0                       ),
        .mdio_in_i                      (1'd0                       ),
        .configuration_vector_i         (configuration_vector       )
    );

assign  serdes_rxdata   =   {2'b00,serdes_txdata[18:10],2'b00,serdes_txdata[8:0]};

always @(posedge rx_mac_clk or negedge rstn) begin
    if(!rstn) begin
        configuration_vector          <=  {3'b011,4'b0011,4'b1000,4'b0100};
    end else begin
        configuration_vector[0]       <=  main_reset;
        configuration_vector[2]       <=  an_enable;
        configuration_vector[4]       <=  an_restart;
        configuration_vector[1]       <=  configuration_vector[1];
        configuration_vector[3]       <=  configuration_vector[3];
        configuration_vector[10:5]    <=  configuration_vector[10:5];
        configuration_vector[13:11]   <=  confi_vector;
        configuration_vector[14]      <=  configuration_vector[14];
    end
end

always @(posedge rx_mac_clk or negedge rstn) begin
    if(!rstn) begin
        ip_link_complete    <=  1'b0;
    end else if(an_enable) begin
        ip_link_complete    <=   status_vector[2] && status_vector[1] && status_vector[0];
    end else begin
        ip_link_complete    <=   status_vector[1] && status_vector[0];    
    end
end

always @(posedge rx_mac_clk or negedge rstn) begin
    if(!rstn) begin
        an_complete     <=  1'b0;
        an_complete_d1  <=  1'b0;
    end else begin
        an_complete     <=  status_vector[2];
        an_complete_d1  <=  an_complete;
    end
end

always @(posedge rx_mac_clk or negedge rstn) begin
    if(!rstn) begin
        ip_link_complete_cnt_rx <=  8'd0;
    end else if(ip_link_complete)begin
        if(ip_link_complete_cnt_rx == 8'h7f) begin
            ip_link_complete_cnt_rx <=  ip_link_complete_cnt_rx;
        end else begin
            ip_link_complete_cnt_rx <=  ip_link_complete_cnt_rx + 8'd1;
        end
    end else begin
        ip_link_complete_cnt_rx <=  8'd0;
    end
end

always @(posedge rx_mac_clk or negedge rstn) begin
    if(!rstn) begin
        rx_mac_rst_n    <=  1'b0;
    end else if(ip_link_complete && (ip_link_complete_cnt_rx == 8'h7f) ) begin
        rx_mac_rst_n    <=  1'b1;
    end else begin
        rx_mac_rst_n    <=  1'b0;  
    end
end

always @(posedge tx_mac_clk or negedge rstn) begin
    if(!rstn) begin
        ip_link_complete_cnt_tx <=  8'd0;
    end else if(ip_link_complete)begin
        if(ip_link_complete_cnt_tx == 8'h7f) begin
            ip_link_complete_cnt_tx <=  ip_link_complete_cnt_tx;
        end else begin
            ip_link_complete_cnt_tx <=  ip_link_complete_cnt_tx + 8'd1;
        end
    end else begin
        ip_link_complete_cnt_tx <=  8'd0;
    end
end

always @(posedge tx_mac_clk or negedge rstn) begin
    if(!rstn) begin
        tx_mac_rst_n    <=  1'b0;
    end else if(ip_link_complete && (ip_link_complete_cnt_tx == 8'h7f) ) begin
        tx_mac_rst_n    <=  1'b1;
    end else begin
        tx_mac_rst_n    <=  1'b0;  
    end
end

always @(posedge rx_mac_clk or negedge rstn) begin
    if(~rstn) begin
        auto_negotiation_complete   <=  1'b0;
    end else if(check_clear) begin
        auto_negotiation_complete   <=  1'b0;
    end else if(auto_negotiation_complete) begin
        auto_negotiation_complete   <=  1'b1;
    end else begin
        if(an_complete && ~an_complete_d1) begin
            auto_negotiation_complete   <=  1'b1;
        end else begin
            auto_negotiation_complete   <=  1'b0;
        end
    end
end

mac_tx_model_word u_mac_tx_model_word(
        .tx_mac_clk                 (tx_mac_clk                 ),
        .tx_mac_rst_n               (tx_mac_rst_n               ),
        
    //CONTROL PORT
        .i_cycle_one_time           (cycle_one_time             ),
        .i_cycle_all_time           (cycle_all_time             ),

        .i_tx_length_hold           (length_hold                ),
        .i_tx_no_pause              (no_pause                   ),
        .i_tx_check_clear           (check_clear                ),

        .tx_running                 (tx_running                 ),

    //IP TX PORT
        .tx_mac_valid               (tx_mac_valid               ),
        .tx_mac_data                (tx_mac_data                ),
        .tx_mac_last                (tx_mac_last                ),
        .tx_mac_ub_invalid          (tx_mac_ub_invalid          ),
        .tx_mac_error               (tx_mac_error               ),
        .tx_mac_ready               (tx_mac_ready               ),

        .tx_pause_req               (tx_pause_req               ),
        .tx_pause_val               (tx_pause_val               ),
        .tx_pause_source_addr       (tx_pause_source_addr       ),
        .tx_statistics_valid        (tx_statistics_valid        ),
        .tx_cnt_er                  (tx_cnt_er                  ),
        .tx_cnt                     (tx_cnt                     )
    );

mac_rx_model_word u_mac_rx_model_word(
        .rx_mac_clk                 (rx_mac_clk                 ),
        .rx_mac_rst_n               (rx_mac_rst_n               ),
                
    //CONTROL PORT
        .i_cycle_one_time           (cycle_one_time             ),
        .i_cycle_all_time           (cycle_all_time             ),

        .i_rx_length_hold           (length_hold                ),
        .i_rx_no_pause              (no_pause                   ),
        .i_rx_check_clear           (check_clear                ),

        .rx_running                 (rx_running                 ),
    //IP TX PORT
        .rx_mac_valid               (rx_mac_valid               ),
        .rx_mac_data                (rx_mac_data                ),
        .rx_mac_last                (rx_mac_last                ),
        .rx_mac_ub_invalid          (rx_mac_ub_invalid          ),
        .rx_mac_error               (rx_mac_error               ),

        .rx_statistics_valid        (rx_statistics_valid        ),
        .rx_statistics_vector       (rx_statistics_vector       ),
        .rx_pause_req               (rx_pause_req               ),
        .rx_pause_val               (rx_pause_val               ),

    //CHECK RESULT
        .rx_data_error              (                           ),
        .rx_cnt_error               (                           ),
        .rx_statistics_error        (                           ),
        .rx_error_all_data          (                           ),

        .rx_data_error_pause        (                           ),
        .rx_length_error_pause      (                           ),
        .rx_statistics_error_pause  (                           ),
        .rx_pause_error             (                           ),
        .rx_error_all_pause         (                           ),

        .rx_error_all               (                           ),
        .rx_error_all_data_check    (error_all_data_check       ),
        .rx_error_all_pause_check   (error_all_pause_check      ),
        .rx_error_all_check         (error_all_check            ),
        .rx_cnt_er                  (rx_cnt_er                  ),
        .rx_cnt                     (rx_cnt                     )
    );

    initial begin
        clk_in = 0;
        forever begin
            #10;
            clk_in = ~clk_in;
        end
    end
    
    initial begin
        pcs_tx_o_fabric_clk = 0;
        forever begin
            #3.2;
            pcs_tx_o_fabric_clk = ~pcs_tx_o_fabric_clk;
        end
    end

    initial begin
        pcs_rx_o_fabric_clk = 0;
        forever begin
            #3.2;
            pcs_rx_o_fabric_clk = ~pcs_rx_o_fabric_clk;
        end
    end

    initial begin
        rstn = 0;
        #100;
        rstn = 1;
    end

task fd_one_time;
    begin
        cycle_one_time          =   1'b1;
        repeat(10) @(posedge rx_mac_clk);
        cycle_one_time          =   1'b0;
        repeat(10) @(posedge rx_mac_clk);
        wait(~tx_running && ~rx_running);
    end
endtask
    
task fd_all_time;
    begin
        cycle_all_time          =   1'b1;
        repeat(10) @(posedge rx_mac_clk);

        while(cycle_cnt < 16'd10) begin
            @(posedge rx_mac_clk);
            if (~rx_running) begin
                cycle_cnt   <=  cycle_cnt + 16'd1;
            end else begin
                cycle_cnt   <=  cycle_cnt;
            end
        end

        @(posedge rx_mac_clk);
        cycle_cnt           <=  16'd0;
        
        cycle_all_time          =   1'b0;

        wait(~tx_running && ~rx_running);
    end
endtask

task functional_testing;
    begin
        wait(rx_mac_rst_n && tx_mac_rst_n);
        repeat(10) @(posedge rx_mac_clk);
    
    //检测传输正常帧 负载在变化
        $display("--------------------NUM1    test  --- start transmitting normal frames : %0tus-------------------",$time);
        fd_one_time;
        $display("--------------------NUM1    test  --- test transmitting normal frames correctly and stop the test : %0tus-------------------",$time);
    
    //检测传输Pause帧
        $display("--------------------NUM2    test  --- start transmitting pause frames : %0tus-------------------",$time);
        fd_one_time;
        $display("--------------------NUM2    test  --- test transmitting pause frames correctly and stop the test : %0tus-------------------",$time);
    
    //检测满载下传输
        $display("--------------------NUM3    test  --- start testing fully loaded transmission : %0tus-------------------",$time);
        //1500负载
        no_pause                =   1'b1;
        length_hold             =   1'b1;
        repeat(10) @(posedge rx_mac_clk);
        fd_all_time;
    
        no_pause                =   1'b0;
        length_hold             =   1'b0;
        repeat(10) @(posedge rx_mac_clk);
        $display("--------------------NUM3    test  --- test fully loaded transmission correctly and stop the test : %0tus-------------------",$time);
    
    //其他功能:重新自协商等
        $display("--------------------NUM4    test  --- start testing other features : %0tus-------------------",$time);
        //main_reset
        main_reset              =   1'b1;
        repeat(40) @(posedge rx_mac_clk);
        wait(~ip_link_complete);
    
        main_reset              =   1'b0;
        repeat(40) @(posedge rx_mac_clk);
    
        wait(rx_mac_rst_n && tx_mac_rst_n);
        repeat(10) @(posedge rx_mac_clk);
        fd_one_time;
    
        //an_restart
        an_restart              =   1'b1;
        repeat(40) @(posedge rx_mac_clk);
        
        an_restart              =   1'b0;
        repeat(40) @(posedge rx_mac_clk);

        wait(rx_mac_rst_n && tx_mac_rst_n);
        repeat(10) @(posedge rx_mac_clk);
        fd_one_time;

        $display("--------------------NUM4    test  --- test other features correctly and stop the test : %0tus-------------------",$time);

        end
    endtask

    initial begin
        wait(error_all_check);
        $display("--------------%0tus:transceiver is incorrect-------------------",$time);
        $finish;
    end

    initial begin
        rstn                =   1'b0;

        an_restart          =   1'b0;
        an_enable           =   1'b1;
        main_reset          =   1'b0;

        length_hold         =   1'd0;
        no_pause            =   1'b0;
        check_clear         =   1'b0;

        basex_or_sgmii      =   1'b0;
        sgmii_mode          =   1'b0;
        basex_link_timer    =   9'd19;
        sgmii_link_timer    =   9'd3;
        confi_vector        =   3'b110;

        cycle_cnt           =   16'd0;

        #100;
        rstn                =   1'b1;
    
        wait(ip_link_complete);
        $display("--------------Auto-Negotiation is complete : %0tus-------------------",$time);

        wait(rx_mac_rst_n && tx_mac_rst_n);
        repeat(10) @(posedge rx_mac_clk);
        $display("--------------MAC Layer Tests-------------------");

//检测全双工模式下的通信
        $display("----------------full duplex tests --- start testing full duplex : %0tus-------------------",$time);

    //2500BASE-X tests
        $display("------------------2500BASE-X tests-------------------");
        functional_testing;

    //2.5G SGMII tests
        $display("------------------2.5G SGMII tests(Media Mode)-------------------");
        basex_or_sgmii      =   1'b1;
        sgmii_mode          =   1'b1;

        an_restart          =   1'b1;
        repeat(40) @(posedge rx_mac_clk);
        
        an_restart          =   1'b0;
        repeat(40) @(posedge rx_mac_clk);

        functional_testing;

        #1000;
        $finish;
    
    end

    initial begin
        $fsdbDumpfile("tb.fsdb");
        $fsdbDumpvars;
    end

endmodule